/*
******************  library jac77 ************************

	The library jac77 includes several routines to
	compute the high atmospheric properties, using the
	jacchia 1977 model.

*/
#include "atmodels.h"
#include "jac77.h"

void isdamo(double sa[], double su[], double sf[], double rjud, double dafr, double gsti,
			double te[], double ad[], double *wmol, double *rhod)
/*

	Purpose:

		The subroutine  isdamo  gives  the  density, molecular

		weight and temperature of the upper atmosphere,  using

		the integrated static version (jacchia 1977)  and  the
            -          -
	    dynamic atmospheric model.
		-       -           --

	Inputs:
		sa[0]
			right ascension of the  point  in  question,  in
			radians (0 to 2.*pi)
		sa[1]
			declination (geocentric latitude) of the  point,
			in radians (-pi to pi).
		sa2[2]
			geocentric altitude of the point in  meters,  in
			the range 90000. to 2000000.m.
		su[0]
			right ascension of the sun at the date, in radi-
			ans (o to 2.*pi).
		su[1]
			sun declination in radians (-pi to pi)
		sf[0]
			daily solar flux at  10.7 cm,  adjusted for
			the distance between the earth and the  sun
			at the date, in 1e-22 w/m/m/hz.
		sf[1]
			averaged daily flux  as defined by jacchia,
			in 1e-22 w/m/m/hz, adjusted for the  earth-
			sun distance.
		sf[2]
			3-hourly planetary geomagnetic index kp, at
			the time (dafr - tau), where tau is given  by
			jacchia's 1977 model.
		rjud
			modified julian date referred to 1950.0
		dafr
			time (ut) of the day, in seconds.
		gsti
			greenwich sideral time, in radians (0 to 2.*pi),
			at the time dafr of the date rjud.

	Outputs:
		te[0]
			mean exospheric temperature above the  point  in 
			question, as defined by equation  20,  in  refe-
			rence (1), in kelvin.
		te[1]
			local temperature around the point, in kelvin.
		ad[0]
			logarithm in base 10 of the He number-density.
		ad[1]
			logarithm in base 10 of the O2 number-density.
		ad[2]
			logarithm in base 10 of the N2 number-density.
		ad[3]
			logarithm in base 10 of the Ar number-density.
		ad[4]
			logarithm in base 10 of the O  number-density.
		ad[5]
			logarithm in base 10 of the H  number-density.
		wmol
			mean-molecular-weight of the atmosphere  at  the
			point in kg/kgmol.
		rhod
			mean-mass-density of the atmosphere, in kg/m/m/m

	Obs:
		He	helium
		O2	molecular oxygen
		N2	molecular nitrogen
		Ar	argon
		O	atomic oxygen
		H	atomic hydrogen

	Subcalls:
		ismade
		divari
		geoaci
		sealat
		semian

	References:
		(1) Jacchia, L. G. "Thermospheric Temperature, Density
		    and Compostion: New Models." Cambridge,  Ma,  SAO,
			1977. (SAO Special Report No 375).

	Authors:
		Bento Silva de Matos   - INPE - S.J.Campos - Brazil
		Valdemir Carrara       - INPE - S.J.Campos - Brazil

	Date:
		Jan. 1985              v. 1.0
		July 2011				C version
*/
{
	static double al[6];

	static double ac[6], th[6];
	static double alco, altu, anac, anut, geac, heig;
	static double rlat, rlon, sudc, texo, tqut, tyfr, weig, yefr;
	static int i;

	yefr   = (rjud + dafr/.86400e5)/.3652422e03;
	tyfr   = yefr - int(yefr);
	geac   = sf[2];
	rlat   = sa[1];
	rlon   = sa[0] - gsti;
	heig   = sa[2];
	altu   = heig/1000.;
	sudc   = su[1];

	ismade(heig, sf, te, al, wmol, rhod);
  
	texo   = te[0];
	
	divari(texo, sa, su, *wmol, al, th, ac);

	for (i = 0; i < 6; i++)
	{
		ad[i] = al[i];
	}
	
	tqut   = th[5];
	
	geoaci(tqut, geac, rlat, rlon, altu, al);

	for (i = 0; i < 6; i++)
	{
		ad[i] += al[i] - ac[i];
	}

	sealat(tyfr, sudc, rlat, altu, al);

	for (i = 0; i < 6; i++)
	{
		ad[i] += al[i];
	}
  
	semian(tyfr, altu, &alco);
  
	for (i = 0; i < 6; i++)
	{
		ad[i] += alco;
	}

	weig   = 0.0;
	anut   = 0.0;

	for (i = 0; i < 6; i++)
	{
		anac  = pow(10.0, ad[i]);
		anut  = anut + anac;
		weig  = weig + _wm[i]*anac;
	}

	*wmol   = weig/anut;
	*rhod   = weig/_avog;

	return;
}
 
void ismade(double altu, double sf[], double te[], double al[],
			double *wmol, double *rhod)
/*

	Purpose:

		The subroutine calculates the atmospheric density  for

		heights from 100 to 2000 km, using the integrated sta-
		                                       -          -
		tic model to compute the atmospheric density (1).
		    -                    -           --
	Inputs:
		altu
			altitude of the point in meters.
		sf[0]
			daily solar flux at  10.7 cm,  adjusted for
			the distance between the earth and the  sun
			at the date, in 1e-22 w/m/m/hz.
		sf[1]
			averaged daily flux  as defined by jacchia,
			in 1e-22 w/m/m/hz, adjusted for the  earth-
			sun distance.

	Outputs:
		te[0]
			mean exospheric temperature above the  point  in 
			question, as defined by equation  20,  in  refe-
			rence (1), in kelvin.
		te[1]
			local temperature around the point, in kelvin.
		al[0]
			logarithm in base 10 of the He number-density.
		al[1]
			logarithm in base 10 of the O2 number-density.
		al[2]
			logarithm in base 10 of the N2 number-density.
		al[3]
			logarithm in base 10 of the Ar number-density.
		al[4]
			logarithm in base 10 of the O  number-density.
		al[5]
			logarithm in base 10 of the H  number-density.
		wmol
			mean-molecular-weight of the atmosphere  at  the
			point in kg/kgmol.
		rhod
			mean-mass-density of the atmosphere, in kg/m/m/m

	Obs:
		He	helium
		O2	molecular oxygen
		N2	molecular nitrogen
		Ar	argon
		O	atomic oxygen
		H	atomic hydrogen

	Subcalls:
		imowei
		temlo

	References:
		(1) Jacchia, L. G. "Thermospheric Temperature, Density
		    and Compostion: New Models." Cambridge,  Ma,  SAO,
			1977. (SAO Special Report No 375).

	Authors:
		Bento Silva de Matos   - INPE - S.J.Campos - Brazil
		Valdemir Carrara       - INPE - S.J.Campos - Brazil

	Date:
		Jan. 1985              v. 1.0
		July 2011				C version
*/
{
	static double cp[7];
	static double argu, fbar, flux, gx, heig, pote, thaf, tx;

	const double to = 188., zx = 125., zo = 90.0, be = 5.5e-5;
  
	flux   = sf[0];
	fbar   = sf[1];
	thaf   = 5.48*pow(fbar, 0.8) + 101.8*pow(flux, .40);
	heig   = altu/1000.;

	imowei(thaf, heig, al, wmol, rhod);

	pote   = 0.0045*(thaf - to);
	argu   = pote/sqrt(1.0 + pote*pote);
	tx     = to + 110.50*atanh(argu);
	gx     = 1.90*(tx - to)/(zx - zo);

	cp[0]  = 2.0*(tx - to)/_pi;
	cp[1]  = gx/cp[0];
	cp[2]  = 1.70*cp[1];
	cp[3]  = 2.0*(thaf - tx)/_pi;
	cp[4]  = gx/cp[3];
	cp[5]  = be*cp[4];
	cp[6]  = tx;

	te[0]  = thaf;
	te[1]  = temlo(heig, cp);
	
	return;
}

void divari(double texo, double sa[], double su[], double wmol,	double al[],
			double th[], double alh[])
/*

	Purpose:

		The subroutine  calculates the  diurnal variations  in
		                                --      ---
		the atmosphere density, as proposed by Jacchia,  using

		the integrated static model of Jacchia (1).
		    -

	Inputs:
		texo
			mean exospheric temperature, in kelvin.
		sa[0]
			right ascension of the  point  in  question,  in
			radians (0 to 2.*pi)
		sa[1]
			declination (geocentric latitude) of the  point,
			in radians (-pi to pi).
		sa[2]
			geocentric altitude of the point in  meters,  in
			the range 90000. to 2000000.m.
		su[0]
			right ascension of the sun at the date, in radi-
			ans (o to 2.*pi).
		su[1]
			sun declination in radians (-pi to pi)
 		wmol
			mean-molecular-weight of the atmosphere  at  the
			point in kg/kgmol.

	Outputs:
		al[0]
			logarithm in base 10 of the He number-density.
		al[1]
			logarithm in base 10 of the O2 number-density.
		al[2]
			logarithm in base 10 of the N2 number-density.
		al[3]
			logarithm in base 10 of the Ar number-density.
		al[4]
			logarithm in base 10 of the O  number-density.
		al[5]
			logarithm in base 10 of the H  number-density.
		th[0]
			exospheric temperature for He, in K.
		th[1]
			exospheric temperature for O2, in K.
		th[2]
			exospheric temperature for N2, in K.
		th[3]
			exospheric temperature for Ar, in K.
		th[4]
			exospheric temperature for  O, in K.
		th[5]
			exospheric temperature for  H, in K.
		alh[]
			same as al[], but using the pseudo exospheric
			temperature for the hydrogen.

	Obs:
		He	helium
		O2	molecular oxygen
		N2	molecular nitrogen
		Ar	argon
		O	atomic oxygen
		H	atomic hydrogen

	Subcalls:
		imowei

	References:
		(1) Jacchia, L. G. "Thermospheric Temperature, Density
		    and Compostion: New Models." Cambridge,  Ma,  SAO,
			1977. (SAO Special Report No 375).

	Authors:
		Bento Silva de Matos   - INPE - S.J.Campos - Brazil
		Valdemir Carrara       - INPE - S.J.Campos - Brazil

	Date:
		Jan. 1985              v. 1.0
		July 2011				C version
*/
{ 
/* common outputs:
       common /stade/ ac(6),th(6)
                ac     same as  al,  but  using the  pseudo
                       exospheric   temperature   for   the
                       hydrogen.
                th(1)  exospheric temperature for he, in k.
                th(2)  exospheric temperature for o2, in k.
                th(3)  exospheric temperature for n2, in k.
                th(4)  exospheric temperature for ar, in k.
                th(5)  exospheric temperature for  o, in k.
                th(6)  exospheric temperature for  h, in k.
*/
	static double an[6];
	static double altu, argu, bish, coar, cola, delt, e1, fhag;
	static double rhod, rlat, soan, teau, temp, weig; 
	static int i;

	rlat   = sa[1];
	delt   = su[1];
	soan   = sa[0] - su[0];
	altu   = sa[2]/1000.;
	argu   = rlat*rlat/1.5707963;
	coar   = cos(argu);
	cola   = cos(rlat);
	e1     = 2.0 + coar*coar;
	teau   = 1.0 + 0.3666069*delt*sin(rlat);

	for (i = 0; i < 5; i++)
	{
		bish  = _rad*(27.0*(wmol/_wm[i] - 1.0) - 35.0);
		argu  = soan + bish;
		fhag  = 0.080*cos(3.0*argu - 1.30899690) + pow(fabs(cos(0.5*argu)), e1);
		th[i] = (0.24*cola*(fhag - 0.5) + teau)*texo;
	}
	
	argu   = soan - 1.0471976;
	fhag   = 0.08*cos(3.*argu - 1.3089969) + pow(fabs(cos(0.5*argu)), e1);
	th[5]  = texo*(teau + 0.24*cola*(fhag - 0.50));
	
	for (i = 0; i < 6; i++)
	{
		temp   = th[i];
		imowei(temp, altu, an, &weig, &rhod);
		al[i]  = an[i];
	}
	 
	for (i = 0; i < 6; i++)
	{
		alh[i]  = an[i];
	}
	
	return;
}

void geoaci(double tqut, double geac, double rlat, double rlon, double altu,
			double dn[])
/*

	Purpose:

		The  subroutine  geoaca  obtains  the variation of the

		atmosphere  number-density,  due  to  the  geomagnetic
		                                           ---
		activity, using the integrated static model.
		--                  -

	Inputs:
		tqut
			exospheric  temperature  for  the  hydrogen
			above the point, in kelvin.
		geac
			the planetary three-hour index, kp, in  the
			range 0. to 9.0
		rlat
			declination (geocentric  latitude)  of  the
			point, in radians (-pi to pi).
		rlon
			local longitude, in radians (-pi to pi).
		altu
			geocentric  altitude  in  km,  in the range
			90. to 2000..

	Outputs:
		dn
			array with the common logarithm  (base 10),
			of the variation in the  number density  of
			the He (helium), O2 (molecular oxygen),  N2
			(molecular nitrogen), Ar (argon), O (atomic
			oxygen) and H (atomic hydrogen),  respecti-
			vely, due to the geomagnetic activity.

	Subcalls:
		imowei

	References:
		(1) Jacchia, L. G. "Thermospheric Temperature, Density
		    and Compostion: New Models." Cambridge,  Ma,  SAO,
			1977. (SAO Special Report No 375).

	Authors:
		Bento Silva de Matos   - INPE - S.J.Campos - Brazil
		Valdemir Carrara       - INPE - S.J.Campos - Brazil

	Date:
		Jan. 1985              v. 1.0
		July 2011				C version
*/
{
	static double auxi, col2, dele, delt, dezh, sen2, rhod, seno, sila, temp, wmol;
	static int i;

	static const double gm[6] =  {-6.30e-05, 1.03e-05, 0.0, 3.07e-05, -4.85e-05, 0.0};

	auxi   = 57.5*geac*(1.0 + 0.027*exp(0.4*geac));
	sila   = sin(rlat);
	seno   = 0.9792*sila + 0.2028*cos(rlat)*cos(rlon - 5.0789081);
	sen2   = seno*seno;
	col2   = 1.0 - sen2;
	delt   = auxi*sen2*sen2;
	temp   = tqut + delt;

	imowei(temp, altu, dn, &wmol, &rhod);

	delt   = 0.01*delt;
	dezh   = 5000.*log(sqrt(1.0 + delt*delt) + delt);
	dele   = 5.2e-04*auxi*col2*col2;
	
	for (i = 0; i < 6; i++)
	{
		dn[i] = dn[i] + gm[i]*dezh + dele;
	}

	return;
}

void imowei(double texo, double altu, double an[], double *wmol, double *rhod)
/*

	Purpose:

		This subroutine uses the  numerically  integrated  mo-
		                                       -
		del (Jacchia, 1977-1) for the molecular weight of  the 
		                              --        ---
		upper  atmosphere.

	Inputs:
		texo
			exospheric temperature, as described in  Jacchia
			(1), in kelvin.
		altu
			geometrical height where  the  molecular  weight
			will be calculated, in km. (90 to 2000km.).

	Outputs:
		an[6]
			common logarithm (base 10) of the He (helium),
			O2 (molecular oxygen), N2 (molecular    nitro-
			gen), Ar (argon), O (atomic oxygen) and H  (a-
			tomic hidrogen) number-density, respectively.
		wmol
			mean molecular weight, in kg/kgmol.
		rhod
			atmospheric density in kg/m/m/m.

	References:
		(1) Jacchia, L. G. "Thermospheric Temperature, Density
		    and Compostion: New Models." Cambridge,  Ma,  SAO,
			1977. (SAO Special Report No 375).

	Authors:
		Bento Silva de Matos   - INPE - S.J.Campos - Brazil
		Valdemir Carrara       - INPE - S.J.Campos - Brazil

	Date:
		Jan. 1985              v. 1.0
		July 2011				C version
*/
{
	static double bn[6], aa[7], al[6];
	static int nc, i;
	static double auxi, denu, fat2, grav, gx, h, hnd5, phis, pote, rhos, rsls, rslt;
	static double soma, step, sumh, summ, sumo, temf, temi, temp, titf, tx, weig, zaux, zend, zini;

	static const double wt[5] = {0.31111111111, 1.4222222222, 0.53333333333,
		1.4222222222, 0.31111111111};
	static const double ci[6] = {28.89122, -2.83071e-02, -6.59924e-03,
		-3.39574e-04, 6.19256e-05, -1.84796e-06};
	static const double ap[6] = {-0.38, 0.0, 0.0, 0.0, 0.0, -0.25};
	static const double frc1 = 5.242e-06, frc2 = 0.20955, frc3 = 0.78110, frc4 = 9.343e-03;
	static const double rhoi = 3.43e-06, rmli = 28.960;
	static const double be = 5.5e-05, zo = 90.0, zx = 125.0, to = 188.0;

	pote    = 0.0045*(texo - to);
	auxi    = pote/sqrt(1. + pote*pote);
	tx      = to + 110.5*atanh(auxi);
	aa[0]   = 2.*(tx - to)/_pi;
	aa[3]   = 2.*(texo - tx)/_pi;
	gx      = 1.9*(tx - to)/(zx - zo);
	aa[1]   = gx/aa[0];
	aa[2]   = 1.7*aa[1];
	aa[4]   = gx/aa[3];
	auxi    = 28.9/pow(texo, 0.25);

	phis    = pow(10., 6.90 + auxi)/2.0e+20;
	hnd5    = (5.94 + auxi)*_al10;
	aa[5]   = be*aa[4];
	aa[6]   = tx;
	nc      = 5;

	step    = 0.05;
	rslt    = 0.;
	zini    = zo;
	zend    = 100.;

	if (altu < zend) zend  = altu;
	
	zaux    = (zend - zini)/4.;
	auxi    = int(zaux/step);
	if (auxi <= 0.) auxi  = 1.;
	step    = zaux/auxi;

	while (fabs(zend - zini) > 0.0001)
	{
		summ   = 0.;
		zaux   = zini + 2.*step;
		auxi   = 1. + zaux/_ra;
		grav   = _go/auxi/auxi;
		zaux   = zini;

		for (i = 0; i < 5; i++)
		{
			h     = zaux - zo;
			weig  = ((((ci[5]*h + ci[4])*h + ci[3])*h + ci[2])*h + ci[1])*h + ci[0];
			summ  = summ + grav*weig/temlo(zaux, aa)*wt[i];
			zaux  = zaux + step;
		}
		
		rslt   = rslt + step*summ;
		zini   = zini + 4.*step;
	}
	
	rhos    = rhoi*exp(-rslt/_rgas);
	h       = zend - zo;
	weig  = ((((ci[5]*h + ci[4])*h + ci[3])*h + ci[2])*h + ci[1])*h + ci[0];
	temf    = temlo(zend, aa);
	denu    = _avog*rhos/rmli*to/temf;
	rhos    = denu*weig;
	fat2    = rhos/rmli;
	an[0]   = log(frc1*fat2);
	an[1]   = log(fat2*(1. + frc2) - denu);
	an[2]   = log(frc3*fat2);
	an[3]   = log(frc4*fat2);
	an[4]   = log(2.*(denu - fat2));
	an[5]   = 0.;

	if (altu > 100.) // go to 800
	{
		step    = 0.05;
		zini    = 100.;
		temi    = temf;
		rslt    = 0.;
		zend    = 140.;

		if (altu < zend) zend  = altu;

		zaux    = (zend - zini)/4.;
		auxi    = int(zaux/step);

		if (auxi <= 0.) auxi  = 1.;

		step    = zaux/auxi;

		while (fabs(zend-zini) > 0.0001)
		{
			zaux   = zini + 2.*step;
			auxi   = 1. + zaux/_ra;
			grav   = _go/auxi/auxi/_rgas;
			summ   = 0.;
			zaux   = zini;

			for (i = 0; i < 5; i++)
			{
				summ  = summ + wt[i]*grav/temlo(zaux, aa);
				zaux  = zaux + step;
			}
			
			rslt   = rslt + step*summ;
			zini   = zini + 4.*step;
		}
		
		temf    = temlo(zend, aa);
		auxi    = log(temi/temf);
		
		for (i = 0; i < 5; i++)
		{
			an[i]  = an[i] - rslt*_wm[i] + auxi*(1 + ap[i]);
		}
		
		if (altu > 140.) // goto 800;  <
		{
			nc      = 6;
			zini    = 140.;
			step    = 5.;
			rslt    = 0.;
			temi    = temf;
			zend    = 500.;

			while (fabs(zend-zini) > 0.0001)
			{
				zaux   = zini + 2.*step;
				auxi   = 1. + zaux/_ra;
				grav   = _go/auxi/auxi/_rgas;
				summ   = 0.;
				zaux   = zini;
		
				for (i = 0; i < 5; i++)
				{
					summ  = summ + grav*wt[i]/temlo(zaux, aa);
					zaux  = zaux + step;
				}
	
				rslt   = rslt + step*summ;
				zini   = zini + 4.*step;
			}
	
			temf    = temlo(zend, aa);
			auxi    = log(temi/temf);
			an[5]   = hnd5;

			for (i = 0; i < 5; i++)
			{
				an[i]  = an[i] - rslt*_wm[i] + auxi*(1 + ap[i]);
			}
	
			if (altu != 500.) //goto 800; <
			{
				if (altu < 500.)
				{
					temi   = temf;
					step   = -5.;
					zini   = 500.;
					zend   = altu;
					zaux   = (zend - zini)/4.;
					auxi   = int(zaux/step);
					if (auxi <= 0.) auxi  = 1.;
					step   = zaux/auxi;

					while (fabs(zend - zini) > 0.0001)
					{
						al[0] = an[0];
						al[1] = an[1] - 0.07*(1. + tanh(0.18*(zini - 111.)))*_al10;
						al[2] = an[2];
						al[3] = an[3];
						al[4] = an[4] - 0.24*exp(-0.009*(zini - 97.7)*(zini - 97.7))*_al10;
						zaux  = zini + 2.*step;
						auxi  = 1. + zaux/_ra;
						grav  = _go/auxi/auxi/_rgas;
						summ  = 0.;
						sumo  = 0.;
						zaux  = zini;
				
						for (i = 0; i < 5; i++)
						{
							summ = summ + grav*wt[i]/temlo(zaux, aa);
							sumo = sumo + exp(al[i]);
							zaux = zaux + step;
						}
				
						zini  = zini + 4.*step;
						rslt  = step*summ;
						temf  = temlo(zini, aa);
						titf  = log(temi/temf);
						temi  = temf;

						for (i = 0; i < 5; i++)
						{
							an[i]= an[i] - rslt*_wm[i] + titf*(1. + ap[i]);
						}
				
						soma  = sumo/exp(an[5])*phis;
						zaux  = zini + 2.*step;
						auxi  = 1. + zaux/_ra;
						grav  = _go/auxi/auxi/_rgas;
						zaux  = zini;
						summ  = 0.;
						sumo  = 0.;
						sumh  = 0.;
				
						for (i = 0; i < 5; i++)
						{
							temp = temlo(zaux, aa);
							summ = summ + wt[i]/temp;
							auxi = wt[i]/sqrt(temp);
							sumo = sumo + soma*auxi;
							sumh = sumh + auxi ;
							zaux = zaux + step;
						}
				
						auxi  = grav*summ*step*_wm[5];
						rslt  = auxi - titf*(1. + ap[5]);
						auxi  = 1000.*step;
						rsls  = sumo*auxi;
						auxi  = sumh*phis*auxi;
						an[5] = an[5] - rslt - rsls - auxi;
					}
				}
				else
				{
					zini   = 500.;
					step   = 2.5;
					rslt   = 0.;
					rsls   = 0.;
					temi   = temf;
					zend   = altu;
					zaux   = (zend - zini)/4.;
					auxi   = int(zaux/step);
					if (auxi <= 0.) auxi  = 1.;
					step   = zaux/auxi;
			
					while (fabs(zend - zini) > 0.0001)
					{
						zaux  = zini + 2.*step;
						auxi  = 1. + zaux/_ra;
						grav  = _go/auxi/auxi/_rgas;
						summ  = 0.;
						sumo  = 0.;
						zaux  = zini;
				
						for (i = 0; i < 5; i++)
						{
							temp = temlo(zaux, aa);
							summ = summ + grav/temp*wt[i];
							sumo = sumo + wt[i]/sqrt(temp);
							zaux = zaux + step;
						}
				
						rslt  = rslt + step*summ;
						rsls  = rsls + 1000.*step*phis*sumo;
						zini  = zini + 4.*step;
					}
			
					temf   = temlo(zend, aa);
					auxi   = log(temi/temf);
			
					for (i = 0; i < 6; i++)
					{
						an[i] += auxi*(1. + ap[i]) - rslt*_wm[i];
					}
			
					an[5]  = an[5] - rsls;
				}
			}
		}
	}
	//  800 continue
	an[1]   = an[1] - 0.07*(1. + tanh(0.18*(zend - 111.)))*_al10;
	auxi    = zend - 97.7;
	an[4]   = an[4] - 0.24*exp(-0.009*auxi*auxi)*_al10;
	weig    = 0.;
	summ    = 0.;
		
	for (i = 0; i < nc; i++)
	{
		auxi   = exp(an[i]);
		an[i]  = an[i]/_al10;
		if (an[i] < 0.) an[i]   = 0.;
		weig   = weig + auxi*_wm[i];
		summ   = summ + auxi;
	}
		
	*wmol    = weig/summ;
	*rhod    = weig/_avog;
		
	return;
}
